package processing

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/resource_model"
	"cloud.o-forge.io/core/oc-lib/models/resources/datacenter"
	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
)

type ProcessingResource struct {
	resource_model.AbstractResource
	CPUs         []*datacenter.CPU `bson:"cpus,omitempty" json:"cp_us,omitempty"`
	GPUs         []*datacenter.GPU `bson:"gpus,omitempty" json:"gp_us,omitempty"`
	RAM          *datacenter.RAM   `bson:"ram,omitempty" json:"ram,omitempty"`
	Storage      uint              `bson:"storage,omitempty" json:"storage,omitempty"`
	Parallel     bool              `bson:"parallel,omitempty" json:"parallel,omitempty"`
	ScalingModel uint              `bson:"scaling_model,omitempty" json:"scaling_model,omitempty"`
	DiskIO       string            `bson:"disk_io,omitempty" json:"disk_io,omitempty"`
}

func (dma *ProcessingResource) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *ProcessingResource) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}

func (d *ProcessingResource) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()
	data.Init(utils.PROCESSING_RESOURCE, caller)
	return data
}
