package shared_workspace

import (
	"encoding/json"

	"cloud.o-forge.io/core/oc-lib/models/utils"
	"cloud.o-forge.io/core/oc-lib/tools"
	"github.com/google/uuid"
)

type SharedWorkspace struct {
	utils.AbstractObject
	Version     string                 `json:"version,omitempty" bson:"version,omitempty" validate:"required"`
	Description string                 `json:"description,omitempty" bson:"description,omitempty"`
	Attributes  map[string]interface{} `json:"attributes,omitempty" bson:"attributes,omitempty"`
	Workspaces  []string               `json:"workspaces,omitempty" bson:"workspaces,omitempty"`
	Workflows   []string               `json:"workflows,omitempty" bson:"workflows,omitempty"`
	Peers       []string               `json:"peers,omitempty" bson:"peers,omitempty"`
	Rules       []string               `json:"rules,omitempty" bson:"rules,omitempty"`
}

func (ao *SharedWorkspace) GetID() string {
	return ao.UUID
}

func (r *SharedWorkspace) GenerateID() {
	if r.UUID == "" {
		r.UUID = uuid.New().String()
	}
}

func (d *SharedWorkspace) GetName() string {
	return d.Name
}

func (d *SharedWorkspace) GetAccessor(caller *tools.HTTPCaller) utils.Accessor {
	data := New()
	data.Init(utils.SHARED_WORKSPACE, caller)
	return data
}

func (dma *SharedWorkspace) Deserialize(j map[string]interface{}) utils.DBObject {
	b, err := json.Marshal(j)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, dma)
	return dma
}

func (dma *SharedWorkspace) Serialize() map[string]interface{} {
	var m map[string]interface{}
	b, err := json.Marshal(dma)
	if err != nil {
		return nil
	}
	json.Unmarshal(b, &m)
	return m
}
